# =============================================================================
# MODULE: utilities.pl -- Perl utility functions
#
# PROGRAMMER: Michel Simard -- TAO
#
# COMMENTS: 
#
#    Copyright (C) 1996, Government of Canada,
#          Center for Information Technology Innovation (CITI),
#          All rights reserved.
#
# =============================================================================

$verbose_stream = STDERR;


# -----------------------------------------------------------------------------
# Strings
# -----------------------------------------------------------------------------

sub Deaccent {
    my ($s) = @_;

    $s =~ tr//AAAAAAACEEEEIIIIDNOOOOOOUUUUYPBaaaaaaaceeeeiiiidnoooooouuuuy/; # Remove accents
    return $s;
}


sub LowerCase {
    my ($s) = @_;

    $s =~ tr/A-Z-/a-z-/;	# Fold in lower case
    return $s;
}


# -----------------------------------------------------------------------------
# Math
# -----------------------------------------------------------------------------
sub Min {
    my @x = @_;
    my ($min, $i);

    for ($i = 0, $min = $x[0]; $i <= $#x; $i++) {
	$min = $x[$i]
	    if ($x[$i] < $min);
    }

    return $min;
}


sub Max {
    my @x = @_;
    my ($max, $i);

    for ($i = 0, $max = $x[0]; $i <= $#x; $i++) {
	$max = $x[$i]
	    if ($x[$i] > $max);
    }

    return $max;
}


sub Between {
    my ($x, $x1, $x2) = @_;

    return ((($x1 <= $x) && ($x <= $x2))
	    ||
	    (($x2 <= $x) && ($x <= $x1)));
}


sub Round {
    my ($x) = @_;
    
    return int($x + 0.5);
}


sub Distance {
    my($x1, $y1, $x2, $y2) = @_;

    return sqrt(($x1 - $x2) ** 2 + ($y1 - $y2) ** 2);
}


$SMALL_VALUE = 0.00001;
sub SegDistance {
    my ($px, $py, $x1, $y1, $x2, $y2) = @_;
    my ($mf, $bf, $mg, $bg, $rx, $ry);

    # Deal with limit cases:
    if (abs($x1 - $x2) <= $SMALL_VALUE) {
	$rx = $x1;
	$ry = $py;
    } elsif (abs($y1 - $y2) <= $SMALL_VALUE) {
	$rx = $px;
	$ry = $y1;

	# Deal with normal case:
    } else {
	# (x1,y1) and (x2,y2) define a line segment 'f'.  We find the slope and
	# intersect of that segment:
	$mf = ($y2 - $y1) / ($x2 - $x1); # Slope of 'f'
	$bf = $y1 - ($mf * $x1);

	# We then compute the slope and intersect of the line 'g', perpendicular
	# to 'f', that goes through (px,py):
	$mg = -1 / $mf;
	$bg = $py - ($mg * $px);

	# We then go on to compute the point of intersection (rx,ry) between 'g'
	# and the line that goes through 'f' (recall that 'f' is just a line
	# segment):
	$rx = ($bg - $bf) / ($mf - $mg);
	$ry = ($mf * $rx) + $bf;
    }

    # If (rx,ry) lies outside of segment 'f', then the distance between
    # (px,py) and 'f' is the minimum of the distance between that point and
    # either (x1,y1) or (x2,y2), otherwise it's just the distance between
    # (rx,ry) and (px,py).

    return ((Between($rx, $x1, $x2) && Between($ry, $y1, $y2)) 
	    ? Distance($px, $py, $rx, $ry)
	    : Min(Distance($px, $py, $x1, $y1), 
		  Distance($px, $py, $x2, $y2)));
}


# -----------------------------------------------------------------------------
# References
# -----------------------------------------------------------------------------
sub NewArray {
    my @array = @_;
    return \@array;
}




# -----------------------------------------------------------------------------
# Verbosity
# -----------------------------------------------------------------------------
sub UnlessSilent {
    my ($format, @args) = @_;

    printf(STDOUT $format, @args)
	unless ($silent);
}


sub Verbose {
    my ($format, @args) = @_;

    printf($verbose_stream $format, @args)
	if ($verbose || $veryverbose);
}



sub VeryVerbose {
    my ($format, @args) = @_;

    printf($verbose_stream $format, @args)
	if ($veryverbose);
}



sub JustVerbose {
    my ($format, @args) = @_;

    printf($verbose_stream $format, @args)
	if ($verbose && !$veryverbose);
}



# -----------------------------------------------------------------------------
# Overwriting
# -----------------------------------------------------------------------------
sub InitOvwrt {
    $previous_ovwrt = "";
}


sub Ovwrt {
    my ($format, @args) = @_;

    my $del_len = length($previous_ovwrt);
    my $new = sprintf(sprintf("%%-%ds", $del_len), sprintf($format, @args));    

    return ("" x $del_len . ($previous_ovwrt = $new));
}



# -----------------------------------------------------------------------------
# Misc
# -----------------------------------------------------------------------------
sub ReadFile {
    my ($file) = @_;

    open(FILE, "<$file") || die "Can't open $file.";
    
    my $size = (stat(FILE))[7];

    read FILE, $text, $size;

    return $text;    
}


sub ReadStream {
    my ($stream) = @_;

    my $size = (stat($stream))[7];

    read $stream, $text, $size;

    return $text;    
}


sub SaveStderr {
    open(SAVEERR, ">&STDERR");
    open(STDERR, ">/dev/null");
}


sub RestoreStderr {
    my $current;
    close(STDERR);
    open(STDERR, ">&SAVEERR");
    $current = select STDERR;
    $| = 1;
    select $current;
}

1;
